#include <Wire.h>
#include <OneWire.h>
#include <EEPROM2.h>
#include <LiquidCrystal_I2C.h>
#include <DallasTemperature.h>
#include <Encoder.h>

#define Water_Level_Sensor 12
#define  WAIT_INTERVAL  15000UL  // интервал задержки включения 


int T1_min = 10;                   //количество подаваемой мощности в нагреватели (0-254)
int del = 200;                     // переменная ожидания между выборами меню
int interval = 100;                // интервал сколько будет длиться цикл while, после чего перейдёт к следующему меню.(кол-во итераций)
int WindowSize = 1000;             //длина окна регулировки мощности

int ventTemp;                    //Переменная для температуры вентиляции
unsigned long windowStartTime;    //переменная для хранения начала цикла окна


unsigned long loopTimeEncoder;    //переменная для опроса энкодера по таймеру

unsigned char clockwise = 0;  //вращение энкодера по часовой стрелке
unsigned char counterclockwise = 0;   //вращение энкодера против часовой
unsigned char encoderButton = 0; //нажатие кнопки энкодера = 0
const int temp_pin = 13;           // К какому пину подключены Далласы

const int K1_pin = 45;          //Пин Клапана К1 холодной воды
const int K2_pin = 46;          //
const int K3_pin = 47;          //
const int K4_pin = 48;          //
const int K5_pin = 49;          
const int K6_pin = 50;
const int PMP1_pin = 51;        //пин насоса в бане

char* now;                      
long oldPosition  = 500;

byte sensorPin[] = {6, 7, 8};
byte ledPin[] = {9, 10, 11}; // светодиоды
const byte SENS = 3;
int level = 0; 
boolean PMP1_mode=false;
unsigned int T1_max; //Выставляемая максимальная температура коллектора
  // объявляем Далласы
  float temp_T1;
  float temp_T2;
  float temp_T3;
  float temp_to_heater;
  float t; // температура на часах DS3231
  
LiquidCrystal_I2C lcd(0x27, 20, 2); //инициализация дисплея

// Инициализация пинов часов 2-RST 3-DAT 4-CLK  
//DS1302 rtc(2, 3, 4);
OneWire oneWire(temp_pin);
DallasTemperature sensors(&oneWire);

// Энкодер
int val;
int PIN_ENCODER_CLK = 52;
int PIN_ENCODER_DT = 18;
int PIN_ENCODER_SW = 53;
Encoder myEnc( PIN_ENCODER_DT, PIN_ENCODER_CLK );


//Часы DS3231
#define DS3231_I2C_ADDRESS 104
byte seconds, minutes, hours, day, date, month, year;
char weekDay[4];
char L1[5]; // запишем в нее уровни воды в коллекторе
int L2; // переменная чтобы читать статус уровня воды в печи

byte tMSB, tLSB;
float temp3231;




void setup() {

  Serial.begin(9600);  // Старт сериал-порта
  Wire.begin();        // старт I2C
  lcd.init();          // Старт дисплея
  lcd.begin(20, 2);
  lcd.backlight();     // Включаем подсветку дисплея
  // красивая "загрузка" программы при старте :)
  lcd.print("Loading");
  delay(500);                // ждем 0.5 секунды
  lcd.print(".");
  delay(500);                // ждем 0.5 секунды
  lcd.print(".");
  delay(500);                // ждем 0.5 секунды
  lcd.print(".");
  delay(1000);               // ждем 1 секунду
  lcd.clear();               //очистка экрана

  pinMode(PIN_ENCODER_SW, INPUT);
  pinMode (PIN_ENCODER_CLK,INPUT);
  pinMode (PIN_ENCODER_DT,INPUT);
  digitalWrite(PIN_ENCODER_SW, HIGH);
  
  pinMode(PMP1_pin, OUTPUT);
  pinMode(K1_pin, OUTPUT);
  pinMode(Water_Level_Sensor, INPUT);
  
  for(int i = 0; i < SENS; i++) {
     pinMode(sensorPin[i], INPUT); 
     pinMode(ledPin[i], OUTPUT);
    }
    
  digitalWrite(K1_pin, HIGH);
  digitalWrite(K2_pin, HIGH);
  digitalWrite(K3_pin, HIGH);
  digitalWrite(K4_pin, HIGH);
  digitalWrite(K5_pin, HIGH);
  digitalWrite(K6_pin, HIGH);
  digitalWrite(PMP1_pin, HIGH);
}


void loop() {
  sensors.requestTemperatures();
  temp_T1 = sensors.getTempCByIndex(0);
  temp_T2 = sensors.getTempCByIndex(1);
  
  //Часы DS3231
  EEPROM_read(1, PMP1_mode);
  
  watchConsole();
  get3231Date();
  //;
  if (temp3231>0){
    t=temp3231;
  } else {
    get3231Temp();
    }
  Serial.print(weekDay); Serial.print(", "); Serial.print(date, DEC); Serial.print("/"); Serial.print(month, DEC); Serial.print("/"); Serial.print(year, DEC); Serial.print(" - ");
  Serial.print(hours, DEC); Serial.print(":"); Serial.print(minutes, DEC); Serial.print(":"); Serial.print(seconds, DEC);
  Serial.print(" - Temp: "); Serial.println(temp3231);

  
  encoder(); //читаем энкодер

    if (clockwise){     //если вращение по часовой стрелке, показываем дату-время
    //delay(del);
    lcd.clear();
    display_time();
    clockwise = 0;
  }

  if (counterclockwise){ //если вращение против часовой, показываем температуру
    //delay(del);
    lcd.clear();
    info1();
    counterclockwise = 0;
  }

  if (encoderButton){  //если нажата кнопка энкодера, то переходим в меню
    //delay(del/2);
    lcd.clear();
    menu();
  }
  EEPROM_read(3, T1_max);

 //Проверка Уровня воды в коллекторе
  for(int i = 0; i < SENS; i++) {
     if(digitalRead(sensorPin[i]) == LOW) {
        digitalWrite(ledPin[i], HIGH);
        level = SENS - i;
     } else {
       digitalWrite(ledPin[i], LOW);       
     }    
  }
    
  switch(level) {
     case 1:
     strcpy(L1, "FULL");
     digitalWrite(K1_pin, HIGH);
      break;
     case 2:{
      strcpy(L1, "50%");
      digitalWrite(K1_pin, LOW);
     }
       break;
     case 3:{
       strcpy(L1, "LOW");
       digitalWrite(K1_pin, LOW);
     }
       break;
     default:
      strcpy(L1, "EMPT");
      break;
  } 


  mode_collector(WAIT_INTERVAL); //управление насосами 
  termostat();    //выполняем функцию термостата

  //delay(del);
}


void encoder(){

  long newPosition = myEnc.read();
  if (newPosition != oldPosition) {
    Serial.print("clockwise= ");
    Serial.println(clockwise);
      if (newPosition >= oldPosition + 7){
      clockwise = 1;
     
    }
    if (newPosition <= oldPosition - 7){
      counterclockwise = 1;
    }
    oldPosition = newPosition;
    Serial.println(newPosition);
  }
  else {
    clockwise = 0;
    counterclockwise = 0;
  }

  if (digitalRead(PIN_ENCODER_SW) == LOW )  {
    //delay(del/2);
    encoderButton = 1;
  }
  if (digitalRead(PIN_ENCODER_SW) == HIGH )  {
    //delay(del/2);
    encoderButton = 0;
  }
}


void menu(){  //функция отображения меню
  
  manual_control();
  T1_max_setup();
  T1_min_setup();
  vent_setup();

}



void mode_collector(unsigned long pause ){
  static unsigned long prevTime = 0;

  if (temp_T1 > T1_max || temp_T2 > T1_max){
    digitalWrite(K4_pin, LOW); //Поскольку норм.открытый закрывает целых 15 сек...
    
    if(millis() - prevTime > pause) {
      prevTime = millis();
      
      if (temp_T1 > temp_T2) { 
      digitalWrite(K6_pin, LOW);
      digitalWrite(PMP1_pin, LOW);
      digitalWrite(K2_pin, LOW);
      }
       else {
        digitalWrite(K6_pin, LOW);
        digitalWrite(PMP1_pin, LOW);
        digitalWrite(K3_pin, LOW);
     } 
    }
  }
    else {
    digitalWrite(PMP1_pin, HIGH);
    digitalWrite(K4_pin, HIGH);
    digitalWrite(K6_pin, LOW);
    digitalWrite(K2_pin, HIGH);
    digitalWrite(K3_pin, HIGH);
    digitalWrite(K5_pin, HIGH);
    }
} 
  
  

void display_time(){  //функция отображения DS3231 даты-времени-температуры 

  delay(del);
  lcd.clear();
 
  lcd.setCursor(0, 0); // Устанавливаем курсор для печати времени в верхней строчке
  lcd.print(weekDay); lcd.print(", "); lcd.print(date, DEC); lcd.print("/"); lcd.print(month, DEC); lcd.print("/"); lcd.print(year, DEC);
  lcd.setCursor(0, 1); 
  lcd.print(hours, DEC); lcd.print(":"); lcd.print(minutes, DEC); 
  //lcd.print(":"); lcd.print(seconds, DEC);
  lcd.print(" Temp: "); lcd.print(t, 2);

  delay(6000);
  lcd.clear();
  Serial.println("done");
  return;
}



void info1(){  //функция отображения температур и насосов
  int i;     //motor status flag
  sensors.requestTemperatures();
  temp_T3 = sensors.getTempCByIndex(2);
  delay(del);
  lcd.clear();
  EEPROM_read(3, T1_max);
    lcd.setCursor(0, 0);
    lcd.print("T3 ");
    lcd.print(temp_T3, 1); 
    //lcd.print( "C");
    
    lcd.setCursor(8, 0);
    lcd.print("T1min=");
    lcd.print(T1_min);

  i=digitalRead(PMP1_pin);    
  if(i==HIGH) {
  lcd.setCursor(0,1);
  lcd.print("PMP ON ");
  }
  else
  {
  lcd.setCursor(0,1);
  lcd.print("PMP OFF");
  }
  
   lcd.setCursor(8, 1);
   lcd.print( "T1max=");
   lcd.print (T1_max);
    
  delay(4000);              //ждем 4 секунды и возврат в основной экран.
  lcd.clear();
  Serial.println("done");
  return;
}


void termostat(){    //функция терморегулятора

  lcd.clear();
  
  lcd.setCursor(0, 0);
  lcd.print("L1 ");  lcd.print(L1);
  
  lcd.setCursor(9, 0);
  lcd.print(temp_T1, 1); 
  lcd.print( " C");

  lcd.setCursor(9, 1);
  lcd.print(temp_T2, 1); 
  lcd.print( " C");
    
// Проверка Уровня воды в печи
   L2 = digitalRead(Water_Level_Sensor); 
   if (L2 == HIGH) { 
     lcd.setCursor(0, 1);
     lcd.print( "L2 FULL");
   }
   else {
     lcd.setCursor(0, 1);
     lcd.print( "L2 LOW " );
   }

//delay (del);
}


void manual_control(){  // функция мануальный контроль насоса
  lcd.clear();
  delay(del);
  //lcd.setCursor(0, 0);
  lcd.print("Manual control");
  delay (1000);
  lcd.clear();
  int x = 0;
  while (1) {      //бесконечный цикл
    delay(del);
    x++;
    if (x > interval) {
      Serial.println("timeout");
      break;
    }
    encoder();
    EEPROM_read(1, PMP1_mode);
    lcd.setCursor(0, 0);
    lcd.print("PMP1_mode=");
    lcd.setCursor(0, 1);
    lcd.print("Val-1 NEXT Val+1");

    if (encoderButton) {
      delay(del);
      lcd.clear(); //очищаем экран
      //Serial.println(PMP1_mode);
      break;
    }
    if (clockwise) {
      delay(del);
      EEPROM_write(1, PMP1_mode + 1);
      lcd.clear();
    }

    if (counterclockwise) {
      delay(del);
      EEPROM_write(1, PMP1_mode - 1);
      lcd.clear();
    }
  }
}

void T1_max_setup(){ //устанавливаем максимально допустимую темп. в кллекторе
  unsigned int T1_max;    //
  EEPROM_read(3, T1_max); //
  lcd.clear();
  delay(del);
  lcd.setCursor(0, 0);
  lcd.print("MaxTemp SETUP");
  //Serial.println("MaxTemp SETUP");
  delay (1000);
  lcd.clear();
  int x = 0;
  while (1) { ////бесконечный цикл
    delay(del);
    x++;
    if (x > interval) {
      //Serial.println("timeout");
      break;
    }
    encoder();
    EEPROM_read(3, T1_max);
    lcd.setCursor(0, 0);
    lcd.print("MaxTemp = ");
    //Serial.print("MaxTemp = ");
    lcd.print(T1_max);
    lcd.setCursor(0, 1);
    lcd.print("Temp-2 NEXT Temp+2");

    if (encoderButton) {
      delay(del);
      lcd.clear(); //очищаем экран
      break;
    }
    if (clockwise) {
      delay(del);
      if (T1_max > 101){
        T1_max = 60;
      }
      EEPROM_write(3, T1_max - 2);
      lcd.clear();
    }

    if (counterclockwise) {
      delay(del);
      if (T1_max < 27){
        T1_max = 60;
      }
      EEPROM_write(3, T1_max + 2);
      lcd.clear();
    }
  }
}

void T1_min_setup() {
  delay(del);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("T1_min SETUP");
  Serial.println("T1_min SETUP");
  delay (1000);
  lcd.clear();
  int x = 0;
  while (1) {
    delay(del);
    x++;
    if (x > interval) {
      lcd.clear();
      Serial.println("timeout");
      break;
    }

    encoder();
    EEPROM_read(4, T1_min);   //читаем из памяти уставку диммера
    lcd.setCursor(0, 0);
    lcd.print("T1_min = ");
    lcd.print(T1_min);
    //Serial.println(T1_min);
    lcd.setCursor(0, 1);
    lcd.print("T1min-1 NEXT T1min+1");

    if (encoderButton) {  //если кнопка энкодера, то выходим
      delay(del);
      lcd.clear(); //очищаем экран
      break;
    }

    if (clockwise) {  //если по часовой, то отнимаем
      delay(del);
      T1_min = T1_min - 1;
      if (T1_min < -10) { //не ниже -10
        T1_min = 0;
      }
      EEPROM_write(4, T1_min - 1);
      lcd.clear();
    }

    if (counterclockwise) {  //если по часовой, то прибавляем
      delay(del);
      T1_min = T1_min + 1;
      if (T1_min > 10) {  //не выше 10
        T1_min = 0;
      }
      EEPROM_write(4, T1_min + 1);
      lcd.clear();
    }
  }
}


void vent_setup(){

  delay(del);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("VENT. SETUP");
  //Serial.println("VENT. SETUP");
  delay (1000);
  lcd.clear();
  int x = 0;
  while (1) {
    delay(del);
    x++;
    if (x > interval) {
      lcd.clear();
      Serial.println("timeout");
      break;
    }

    encoder();
    EEPROM_read(7, ventTemp);   //читаем из памяти уставку диммера
    if (ventTemp > 43 || ventTemp < 29){ //если там чушь, то включаем 1 режим.
      ventTemp = 39;
    }
    lcd.setCursor(0, 0);
    lcd.print("Vent.Temp = ");
    lcd.print(ventTemp);
    Serial.print("Vent.Temp = ");
    Serial.println(ventTemp);

    lcd.setCursor(0, 1);
    lcd.print("<--- NEXT --->");

    if (encoderButton) {  //если кнопка энкодера, то выходим
      delay(del);
      lcd.clear(); //очищаем экран
      break;
    }

    if (clockwise) {  //если по часовой, то прибавляем мощность на 5%
      delay(del);
      ventTemp --;
      if (ventTemp < 30) { //не ниже 10%
        ventTemp = 42;
      }
      EEPROM_write(7, ventTemp);
      lcd.clear();
    }

    if (counterclockwise) {  //если по часовой, то прибавляем мощность на 5%
      delay(del);
      ventTemp ++;
      if (ventTemp > 42) { //не ниже 10%
        ventTemp = 30;
      }
      EEPROM_write(7, ventTemp);
      //Serial.println(ventTemp);
      lcd.clear();
    }
  }
}


//Часы DS3231

// Convert normal decimal numbers to binary coded decimal
byte decToBcd(byte val)
{
  return ( (val/10*16) + (val%10) );
}

void watchConsole()
{
  if (Serial.available()) {      // Look for char in serial queue and process if found
    if (Serial.read() == 84) {      //If command = "T" Set Date
      set3231Date();
      get3231Date();
      Serial.println(" ");
    }
  }
}
  
void set3231Date()
{ 
//T(sec)(min)(hour)(dayOfWeek)(dayOfMonth)(month)(year)
//T(00-59)(00-59)(00-23)(1-7)(01-31)(01-12)(00-99)
//Example: 2015-07-09 @ 19:57:11 for the 3rd day of the week -> T1157193090715

  seconds = (byte) ((Serial.read() - 48) * 10 + (Serial.read() - 48)); // Use of (byte) type casting and ascii math to achieve result.  
  minutes = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  hours   = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  day     = (byte) (Serial.read() - 48);
  date    = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  month   = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  year    = (byte) ((Serial.read() - 48) *10 +  (Serial.read() - 48));
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0x00);
  Wire.write(decToBcd(seconds));
  Wire.write(decToBcd(minutes));
  Wire.write(decToBcd(hours));
  Wire.write(decToBcd(day));
  Wire.write(decToBcd(date));
  Wire.write(decToBcd(month));
  Wire.write(decToBcd(year));
  Wire.endTransmission();
}


void get3231Date()
{
  // send request to receive data starting at register 0
  Wire.beginTransmission(DS3231_I2C_ADDRESS); // 104 is DS3231 device address
  Wire.write(0x00); // start at register 0
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDRESS, 7); // request seven bytes

  if(Wire.available()) { 
    seconds = Wire.read(); // get seconds
    minutes = Wire.read(); // get minutes
    hours   = Wire.read();   // get hours
    day     = Wire.read();
    date    = Wire.read();
    month   = Wire.read(); //temp month
    year    = Wire.read();
       
    seconds = (((seconds & B11110000)>>4)*10 + (seconds & B00001111)); // convert BCD to decimal
    minutes = (((minutes & B11110000)>>4)*10 + (minutes & B00001111)); // convert BCD to decimal
    hours   = (((hours & B00110000)>>4)*10 + (hours & B00001111)); // convert BCD to decimal (assume 24 hour mode)
    day     = (day & B00000111); // 1-7
    date    = (((date & B00110000)>>4)*10 + (date & B00001111)); // 1-31
    month   = (((month & B00010000)>>4)*10 + (month & B00001111)); //msb7 is century overflow
    year    = (((year & B11110000)>>4)*10 + (year & B00001111));
  }
  else {
    //oh noes, no data!
  }
  
  switch (day) {
    case 1:
      strcpy(weekDay, "Sun");
      break;
    case 2:
      strcpy(weekDay, "Mon");
      break;
    case 3:
      strcpy(weekDay, "Tue");
      break;
    case 4:
      strcpy(weekDay, "Wed");
      break;
    case 5:
      strcpy(weekDay, "Thu");
      break;
    case 6:
      strcpy(weekDay, "Fri");
      break;
    case 7:
      strcpy(weekDay, "Sat");
      break;
  }
}

float get3231Temp(){


  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0x11);
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDRESS, 2);

  if(Wire.available()) {
    tMSB = Wire.read(); //2's complement int portion
    tLSB = Wire.read(); //fraction portion

    temp3231 = (tMSB & B01111111); //do 2's math on Tmsb
    temp3231 += ( (tLSB >> 6) * 0.25 ); //only care about bits 7 & 8
  }
  else {
    //oh noes, no data!
  }

  return temp3231;
}

//
